package envoy.client.ui;

import java.awt.Color;
import java.awt.Component;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Iterator;
import java.util.List;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ListModel;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.border.MatteBorder;

import envoy.client.EnvoyClient;
import envoy.schema.Message;

import javax.swing.ListSelectionModel;
import javax.swing.UIManager;

import java.awt.Rectangle;
import java.awt.geom.RoundRectangle2D;
import java.awt.ComponentOrientation;
import java.awt.Point;
import java.awt.Dimension;
import javax.swing.border.BevelBorder;

/**
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>ChatWindow.java</strong><br>
 * Created: <strong>28 Sep 2019</strong><br>
 * Author: <strong>Maximilian K&aumlfer &amp; Kai S. K. Engelbart &amp; Leon Hofmeister</strong>
 */
public class ChatWindow extends JFrame {

	private static final long serialVersionUID = 6865098428255463649L;

	private JPanel contentPane = new JPanel();
	private EnvoyClient envoyClient = new EnvoyClient();

	public DefaultListModel<String> listModel = new DefaultListModel<>();
	public ChatWindow() {
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setBounds(100, 100, 600, 800);
		setTitle("Envoy");
		setLocationRelativeTo(null);

		contentPane.setBackground(new Color(0, 0, 0));
		contentPane.setForeground(Color.white);
		contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
		setContentPane(contentPane);
		GridBagLayout gbl_contentPane = new GridBagLayout();
		gbl_contentPane.columnWidths = new int[] { 1, 1, 1 };
		gbl_contentPane.rowHeights = new int[] { 1, 1, 1 };
		gbl_contentPane.columnWeights = new double[] { 0.3, 1.0, 0.1 };
		gbl_contentPane.rowWeights = new double[] { 0.05, 1, 0.07 };
		contentPane.setLayout(gbl_contentPane);

		
		// Chat------------------
		List<Message> testMessages = List.of(envoyClient.createMessage("UserA", "UserB", "Das ist eine Testnachricht."), envoyClient.createMessage("UserB", "UserA", "Das ist die Antwort auf die Testnachricht."));
		
		JList<String> elementList = new JList<>();
		elementList.setFocusTraversalKeysEnabled(false);
		
		elementList.setSelectionForeground(new Color(255, 255, 255));
		elementList.setSelectionBackground(new Color(102, 0, 153));
		elementList.setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
		elementList.setForeground(new Color(255, 255, 255));
		elementList.setBackground(new Color(51, 51, 51));
		
		
		
		elementList.setModel(listModel);
		elementList.setFont( new Font("Arial", Font.PLAIN, 17));
		elementList.setFixedCellHeight(60);
		elementList.setBorder(new EmptyBorder(5, 5, 5, 5));
		
		
		//Only temporary for the predefined messages in the List
		for (int i = 0; i < testMessages.size(); i++) {
			listModel.addElement("<html>" + "<p style=\"color:#d2d235\"> <b> <small>" + getSenderElement(testMessages.get(i)) + "</b> </small>" + "<br>" + "<p style=\"color:white\">" + getFirstContentElement(testMessages.get(i)) + "</span></html>");
		}
		
		
		
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setForeground(new Color(0, 0, 0));
		scrollPane.setBackground(new Color(51, 51, 51));
		scrollPane.setViewportView(elementList);
		scrollPane.setBorder(null);
		
		GridBagConstraints gbc_scrollPane = new GridBagConstraints();
		gbc_scrollPane.fill = GridBagConstraints.BOTH;
		gbc_scrollPane.gridwidth  = 2;
		gbc_scrollPane.gridx = 1;
		gbc_scrollPane.gridy = 1;
		

		gbc_scrollPane.insets = new Insets(10, 10, 10, 10);
		
		contentPane.add(scrollPane, gbc_scrollPane);
		
		
		// Message enter field-----------------
		JTextArea messageEnterTextfield = new JTextArea();
		messageEnterTextfield.setCaretColor(new Color(255, 255, 255));
		messageEnterTextfield.setForeground(new Color(255, 255, 255));
		messageEnterTextfield.setBackground(new Color(51, 51, 51));
		messageEnterTextfield.setLineWrap(true);
		messageEnterTextfield.setBorder(null);
		messageEnterTextfield.setFont( new Font("Arial", Font.PLAIN, 17));
		messageEnterTextfield.setBorder(new EmptyBorder(5, 5, 5, 5));

		GridBagConstraints gbc_moveSelectionMessageEnterTextfield = new GridBagConstraints();
		gbc_moveSelectionMessageEnterTextfield.fill = GridBagConstraints.BOTH;
		gbc_moveSelectionMessageEnterTextfield.gridx = 1;
		gbc_moveSelectionMessageEnterTextfield.gridy = 2;

		gbc_moveSelectionMessageEnterTextfield.insets = new Insets(10, 10, 10, 10);

		contentPane.add(messageEnterTextfield, gbc_moveSelectionMessageEnterTextfield);

		
		
		
		// Post Button-----------------
		JButton postButton = new JButton("Post");
		postButton.setForeground(new Color(255, 255, 255));
		postButton.setBackground(new Color(102, 51, 153));
		postButton.setBorderPainted(false);

		GridBagConstraints gbc_moveSelectionPostButton = new GridBagConstraints();

		gbc_moveSelectionPostButton.fill = GridBagConstraints.BOTH;
		gbc_moveSelectionPostButton.gridx = 2;
		gbc_moveSelectionPostButton.gridy = 2;

		gbc_moveSelectionPostButton.insets = new Insets(10, 10, 10, 10);


		contentPane.add(postButton, gbc_moveSelectionPostButton);

		postButton.addActionListener((evt) -> {
			if (!messageEnterTextfield.getText().isEmpty()) try {
				envoyClient.sendMessage(envoyClient.createMessage("Kai", "Maxi", messageEnterTextfield.getText()));
				addMessageToChat("Du", messageEnterTextfield.getText());
				messageEnterTextfield.setText("");
			} catch (Exception e) {
				JOptionPane.showMessageDialog(this,
						"An exception occured while sending a message. See the log for more details.",
						"Exception occured",
						JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}
		});
	}
	
	/**
	 * gets the Sender from the Message
	 * @param message
	 * @return
	 */
	public String getSenderElement (Message message) {
		return message.getMetaData().getSender();
	}
	
	/**
	 * gets the First Content Element(text) from the Message
	 * @param message
	 * @return
	 */
	public String getFirstContentElement (Message message) {
		return message.getContent().get(0).getText();
	}
	
	/**
	 * adds the Sender and the First Content Element(text) of the new Message to the listModel
	 * @param message
	 * @return
	 */
	public void addMessageToChat(String Sender, String Content) {
		listModel.addElement("<html>" + "<p style=\"color:#d2d235\"> <b> <small>" + Sender + "</b> </small>" + "<br>" + "<p style=\"color:white\">" + Content +  "</span></html>");
	}
	
	
	public static void main(String[] args) {
		EventQueue.invokeLater(() -> {
			try {
				ChatWindow frame = new ChatWindow();
				frame.setVisible(true);
			} catch (Exception e) {
				e.printStackTrace();
			}
		});
	}
}